@@ -54,53 +54,29 @@
 
 #include <libdefs.h>
 
-#ifdef ENABLE_HAVAL
+#if defined(ENABLE_HAVAL)
 
 #include <mhash_haval.h>
-/* #include <assert.h> */
 
 #define HAVAL_VERSION	1
 
-/* Platform-specific settings */
-
-/* #define HARDWARE_ROTATIONS */
-/*#pragma optimize( "", off ) */
-/*#pragma option -Od */
-
-
 /* Nonlinear F functions */
 
-/*#define F1(X6, X5, X4, X3, X2, X1, X0) \
-	((X1) & (X4) ^ (X2) & (X5) ^ (X3) & (X6) ^ (X0) & (X1) ^ (X0))*/
 #define F1(X6, X5, X4, X3, X2, X1, X0) \
 	(((X1) & ((X4) ^ (X0))) ^ ((X2) & (X5)) ^ ((X3) & (X6)) ^ (X0))
 
-/*#define F2(X6, X5, X4, X3, X2, X1, X0) \
-	((X1) & (X2) & (X3) ^ (X2) & (X4) & (X5) ^ \
-	(X1) & (X2) ^ (X1) & (X4) ^ (X2) & (X6) ^ (X3) & (X5) ^ \
-	(X4) & (X5) ^ (X0) & (X2) ^ (X0))*/
 #define F2(X6, X5, X4, X3, X2, X1, X0) \
 	(((X2) & (((X1) & (~(X3))) ^ ((X4) & (X5)) ^ (X6) ^ (X0))) ^ \
 	(((X4) & ((X1) ^ (X5))) ^ ((X3) & (X5)) ^ (X0)))
 
-/*#define F3(X6, X5, X4, X3, X2, X1, X0) \
-	((X1) & (X2) & (X3) ^ (X1) & (X4) ^ (X2) & (X5) ^ (X3) & (X6) ^ (X0) &
-(X3) ^ (X0))*/
 #define F3(X6, X5, X4, X3, X2, X1, X0) \
 	(((X3) & (((X1) & (X2)) ^ (X6) ^ (X0))) ^ ((X1) & (X4)) ^ \
 	((X2) & (X5)) ^ (X0))
 
-/*#define F4(X6, X5, X4, X3, X2, X1, X0) \
-	((X1) & (X2) & (X3) ^ (X2) & (X4) & (X5) ^ (X3) & (X4) & (X6) ^ \
-	(X1) & (X4) ^ (X2) & (X6) ^ (X3) & (X4) ^ (X3) & (X5) ^ \
-	(X3) & (X6) ^ (X4) & (X5) ^ (X4) & (X6) ^ (X0) & (X4) ^(X0))*/
 #define F4(X6, X5, X4, X3, X2, X1, X0) \
 	(((X4) & (((~(X2)) & (X5)) ^ ((X3) | (X6)) ^ (X1) ^ (X0))) ^ \
 	((X3) & (((X1) & (X2)) ^ (X5) ^ (X6))) ^ ((X2) & (X6)) ^ (X0))
 
-/*#define F5(X6, X5, X4, X3, X2, X1, X0) \
-	((X1) & (X4) ^ (X2) & (X5) ^ (X3) & (X6) ^ \
-	(X0) & (X1) & (X2) & (X3) ^ (X0) & (X5) ^ (X0))*/
 #define F5(X6, X5, X4, X3, X2, X1, X0) \
 	(((X1) & ((X4) ^ ((X0) & (X2) & (X3)))) ^ \
 	(((X2) ^ (X0)) & (X5)) ^ ((X3) & (X6)) ^ (X0))
@@ -108,11 +84,6 @@
 
 #define ROTR(v,n)   (((v) >> ((mutils_word32)(n))) | ((v) << (32 - (mutils_word32)(n))))
 
-/* This macro did not work with my gcc, so I removed it.
- * --nikos
- * #define ROTR(v, n) (rot_tmp = (v), (rot_tmp >> (n)) | (rot_tmp << (32 - (n))) )
- */
-
 /*
 The following macro would generate better code on an Intel platform,
 however it is never used there, and this code when used in e.g
@@ -138,26 +109,16 @@ Therefore, allow for unaligned access and move the bytes by hand.
 This assumes that out and in do not overlap, which is the case in
 practice here.
 */
-#define BYTE_REVERSE(out, in, n) \
-{ \
-	register mutils_word8 i; \
-	for (i = 0; i < n; i++) { \
-                ((byte*)out)[4*i+3] = ((byte*)in)[4*i+0];\
-                ((byte*)out)[4*i+2] = ((byte*)in)[4*i+1];\
-                ((byte*)out)[4*i+1] = ((byte*)in)[4*i+2];\
-                ((byte*)out)[4*i+0] = ((byte*)in)[4*i+3];\
-	} \
-}
 
-static void havalTransform3 (mutils_word32 E[8], const byte D[128], mutils_word32 T[8])
+static void havalTransform3 (mutils_word32 E[8], __const mutils_word32 D[32], mutils_word32 T[8])
 {
+	mutils_word32 *W;
 
-#ifndef WORDS_BIGENDIAN
-	mutils_word32 *W = (mutils_word32 *)D;
-#else  /* !LITTLE_ENDIAN */
-	mutils_word32 W[32];
-	BYTE_REVERSE(W, D, 32);
-#endif /* ?LITTLE_ENDIAN */
+#if defined(WORDS_BIGENDIAN)
+	W = mutils_word32nswap(D, 32, MUTILS_FALSE);
+#else  /* LITTLE ENDIAN */
+	W = (mutils_word32 *) D;
+#endif /* Byte re-ordering */
 
 	/* PASS 1: */
 
@@ -274,17 +235,21 @@ static void havalTransform3 (mutils_word32 E[8], const byte D[128], mutils_word3
 	E[1] += T[1] = ROTR (F3 (T[0], T[3], T[4], T[5], T[6], T[7], T[2]), 7) + ROTR (T[1], 11) + W[ 5] + 0xAFD6BA33UL;
 	E[0] += T[0] = ROTR (F3 (T[7], T[2], T[3], T[4], T[5], T[6], T[1]), 7) + ROTR (T[0], 11) + W[ 2] + 0x6C24CF5CUL;
 
+#if defined(WORDS_BIGENDIAN)
+	mutils_free(W);
+#endif
 } /* havalTransform3 */
 
 
-static void havalTransform4 (mutils_word32 E[8], const byte D[128], mutils_word32 T[8])
+static void havalTransform4 (mutils_word32 E[8], __const mutils_word32 D[32], mutils_word32 T[8])
 {
-#ifndef WORDS_BIGENDIAN
-	mutils_word32 *W = (mutils_word32 *)D;
-#else  /* !LITTLE_ENDIAN */
-	mutils_word32 W[32];
-	BYTE_REVERSE(W, D, 32);
-#endif /* ?LITTLE_ENDIAN */
+	mutils_word32 *W;
+
+#if defined(WORDS_BIGENDIAN)
+	W = mutils_word32nswap(D, 32, MUTILS_FALSE);
+#else  /* LITTLE ENDIAN */
+	W = (mutils_word32 *) D;
+#endif /* Byte re-ordering */
 
 	/* PASS 1: */
 
@@ -438,17 +403,21 @@ static void havalTransform4 (mutils_word32 E[8], const byte D[128], mutils_word3
 	E[1] += T[1] = ROTR (F4 (T[0], T[6], T[2], T[7], T[4], T[3], T[5]), 7) + ROTR (T[1], 11) + W[16] + 0x6EEF0B6CUL;
 	E[0] += T[0] = ROTR (F4 (T[7], T[5], T[1], T[6], T[3], T[2], T[4]), 7) + ROTR (T[0], 11) + W[13] + 0x137A3BE4UL;
 
+#if defined(WORDS_BIGENDIAN)
+	mutils_free(W);
+#endif
 } /* havalTransform4 */
 
 
-static void havalTransform5 (mutils_word32 E[8], const byte D[128], mutils_word32 T[8])
+static void havalTransform5 (mutils_word32 E[8], __const mutils_word32 D[32], mutils_word32 T[8])
 {
-#ifndef WORDS_BIGENDIAN
-	mutils_word32 *W = (mutils_word32 *)D;
-#else  /* !LITTLE_ENDIAN */
-	mutils_word32 W[32];
-	BYTE_REVERSE(W, D, 32);
-#endif /* ?LITTLE_ENDIAN */
+	mutils_word32 *W;
+
+#if defined(WORDS_BIGENDIAN)
+	W = mutils_word32nswap(D, 32, MUTILS_FALSE);
+#else  /* LITTLE ENDIAN */
+	W = (mutils_word32 *) D;
+#endif /* Byte re-ordering */
 
 	/* PASS 1: */
 
@@ -640,25 +609,28 @@ static void havalTransform5 (mutils_word32 E[8], const byte D[128], mutils_word3
 	E[1] += T[1] = ROTR (F5 (T[4], T[7], T[2], T[0], T[6], T[5], T[3]), 7) + ROTR (T[1], 11) + W[25] + 0xC1A94FB6UL;
 	E[0] += T[0] = ROTR (F5 (T[3], T[6], T[1], T[7], T[5], T[4], T[2]), 7) + ROTR (T[0], 11) + W[15] + 0x409F60C4UL;
 
+#if defined(WORDS_BIGENDIAN)
+	mutils_free(W);
+#endif
 } /* havalTransform5 */
 
-int havalInit256( havalContext *hcp) {
+mutils_error havalInit256( havalContext *hcp) {
 	return havalInit(hcp, 3, 256);
 }
-int havalInit224( havalContext *hcp) {
+mutils_error havalInit224( havalContext *hcp) {
 	return havalInit(hcp, 3, 224);
 }
-int havalInit192( havalContext *hcp) {
+mutils_error havalInit192( havalContext *hcp) {
 	return havalInit(hcp, 3, 192);
 }
-int havalInit160( havalContext *hcp) {
+mutils_error havalInit160( havalContext *hcp) {
 	return havalInit(hcp, 3, 160);
 }
-int havalInit128( havalContext *hcp) {
+mutils_error havalInit128( havalContext *hcp) {
 	return havalInit(hcp, 3, 128);
 }
 
-mutils_error havalInit (havalContext *hcp, unsigned int passes, unsigned int hashLength)
+mutils_error havalInit (havalContext *hcp, mutils_word32 passes, mutils_word32 hashLength)
 {
 	if (hcp == NULL)
 	{
@@ -674,7 +646,8 @@ mutils_error havalInit (havalContext *hcp, unsigned int passes, unsigned int has
 		hashLength != 160 &&
 		hashLength != 192 &&
 		hashLength != 224 &&
-		hashLength != 256) {
+		hashLength != 256)
+	{
 		return(-MUTILS_INVALID_SIZE); /* invalid hash length */
 	}
 	/* properly initialize HAVAL context: */
@@ -693,29 +666,30 @@ mutils_error havalInit (havalContext *hcp, unsigned int passes, unsigned int has
 } /* havalInit */
 
 
-mutils_error havalUpdate (havalContext *hcp, const byte *dataBuffer, size_t dataLength)
+mutils_error havalUpdate (havalContext *hcp, __const mutils_word8 *dataBuffer, mutils_word32 dataLength)
 {
-	if (hcp == NULL) {
+	if (hcp == NULL)
+	{
 		return(-MUTILS_INVALID_INPUT_BUFFER); /* bad context */
 	}
-	if (dataBuffer == NULL || dataLength == 0) {
+	if (dataBuffer == NULL || dataLength == 0)
+	{
 		return(MUTILS_OK); /* nothing to do */
 	}
 
-/*	assert (hcp->occupied < 128);  invariant */
-
 	/* update bit count: Extra parentheses for Borland C++ --Tines*/
-	if ( ((mutils_word32)dataLength << 3) > (0xFFFFFFFFUL - hcp->bitCount[0]) ) {
+	if ( ((mutils_word32)dataLength << 3) > (0xFFFFFFFFUL - hcp->bitCount[0]) )
+	{
 		hcp->bitCount[1]++;
 	}
 	hcp->bitCount[0] += (mutils_word32)dataLength << 3;
 	
 	/* if the data buffer is not enough to complete */
 	/* the context data block, just append it: */
-	if (hcp->occupied + (mutils_word32)dataLength < 128) { /* caveat: typecast avoids 16-bit overflow */
+	if (hcp->occupied + (mutils_word32)dataLength < 128)
+	{ /* caveat: typecast avoids 16-bit overflow */
 		mutils_memcpy (&hcp->block[hcp->occupied], dataBuffer, dataLength);
 		hcp->occupied += dataLength;
-/*		assert (hcp->occupied < 128); */
 		return(MUTILS_OK); /* delay processing */
 	}
 
@@ -727,30 +701,33 @@ mutils_error havalUpdate (havalContext *hcp, const byte *dataBuffer, size_t data
 	switch (hcp->passes) {
 	case 3:
 		/* process the completed context data block: */
-		havalTransform3 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform3 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		/* process data in chunks of 128 bytes: */
-		while (dataLength >= 128) {
-			havalTransform3 (hcp->digest, dataBuffer, hcp->temp);
+		while (dataLength >= 128)
+		{
+		  havalTransform3 (hcp->digest, (mutils_word32 *) dataBuffer, hcp->temp);
 			dataBuffer += 128;
 			dataLength -= 128;
 		}
 		break;
 	case 4:
 		/* process the completed context data block: */
-		havalTransform4 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform4 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		/* process data in chunks of 128 bytes: */
-		while (dataLength >= 128) {
-			havalTransform4 (hcp->digest, dataBuffer, hcp->temp);
+		while (dataLength >= 128)
+		{
+		  havalTransform4 (hcp->digest, (mutils_word32 *) dataBuffer, hcp->temp);
 			dataBuffer += 128;
 			dataLength -= 128;
 		}
 		break;
 	case 5:
 		/* process the completed context data block: */
-		havalTransform5 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform5 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		/* process data in chunks of 128 bytes: */
-		while (dataLength >= 128) {
-			havalTransform5 (hcp->digest, dataBuffer, hcp->temp);
+		while (dataLength >= 128)
+		{
+		  havalTransform5 (hcp->digest, (mutils_word32 *) dataBuffer, hcp->temp);
 			dataBuffer += 128;
 			dataLength -= 128;
 		}
@@ -761,12 +738,10 @@ mutils_error havalUpdate (havalContext *hcp, const byte *dataBuffer, size_t data
 	mutils_memcpy (hcp->block, dataBuffer, dataLength);
 	hcp->occupied = dataLength; /* < 128 */
 	
-/*	assert (hcp->occupied < 128); */
 	return(MUTILS_OK); /* OK */
 } /* havalUpdate */
 
-
-mutils_error havalFinal (havalContext *hcp, byte *digest)
+mutils_error havalFinal (havalContext *hcp, mutils_word8 *digest)
 {
 	mutils_word32 w;
 
@@ -774,7 +749,8 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 	{
 	  return(-MUTILS_INVALID_INPUT_BUFFER); /* bad context */
 	}
-	if (digest == NULL) {
+	if (digest == NULL)
+	{
 	  return(-MUTILS_INVALID_OUTPUT_BUFFER); /* bad digest buffer */
 	}
 
@@ -784,56 +760,61 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 	hcp->block[hcp->occupied] = 0x01; /* corrected from 0x80 */
 
 	/* pad the message with null bytes to make it 944 (mod 1024) bits long: */
-	if (hcp->occupied++ >= 118) {
+	if (hcp->occupied++ >= 118)
+	{
 		/* no room for tail data on the current context block */
 		mutils_bzero (&hcp->block[hcp->occupied], 128 - hcp->occupied);
 		/* process the completed context data block: */
 		switch (hcp->passes) {
 		case 3:
-			havalTransform3 (hcp->digest, hcp->block, hcp->temp);
+		  havalTransform3 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 			break;
 		case 4:
-			havalTransform4 (hcp->digest, hcp->block, hcp->temp);
+		  havalTransform4 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 			break;
 		case 5:
-			havalTransform5 (hcp->digest, hcp->block, hcp->temp);
+		  havalTransform5 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 			break;
 		}
 		mutils_bzero (hcp->block, 118);
-	} else {
+	}
+	else
+	{
 		mutils_bzero (&hcp->block[hcp->occupied], 118 - hcp->occupied);
 	}
 	/* append tail data and process last (padded) message block: */
-	hcp->block[118] = (byte)(
+	hcp->block[118] = (mutils_word8)(
 		((hcp->hashLength & 0x03U) << 6) |
 		((hcp->passes     & 0x07U) << 3) |
 		(HAVAL_VERSION           & 0x07U));
-	hcp->block[119] = (byte) (hcp->hashLength >> 2);
+	hcp->block[119] = (mutils_word8) (hcp->hashLength >> 2);
 	w = hcp->bitCount[0];
-	hcp->block[120] = (byte)(w);
-	hcp->block[121] = (byte)(w >>  8);
-	hcp->block[122] = (byte)(w >> 16);
-	hcp->block[123] = (byte)(w >> 24);
+	hcp->block[120] = (mutils_word8)(w);
+	hcp->block[121] = (mutils_word8)(w >>  8);
+	hcp->block[122] = (mutils_word8)(w >> 16);
+	hcp->block[123] = (mutils_word8)(w >> 24);
 	w = hcp->bitCount[1];
-	hcp->block[124] = (byte)(w);
-	hcp->block[125] = (byte)(w >>  8);
-	hcp->block[126] = (byte)(w >> 16);
-	hcp->block[127] = (byte)(w >> 24);
-	switch (hcp->passes) {
+	hcp->block[124] = (mutils_word8)(w);
+	hcp->block[125] = (mutils_word8)(w >>  8);
+	hcp->block[126] = (mutils_word8)(w >> 16);
+	hcp->block[127] = (mutils_word8)(w >> 24);
+	switch (hcp->passes)
+	{
 	case 3:
-		havalTransform3 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform3 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		break;
 	case 4:
-		havalTransform4 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform4 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		break;
 	case 5:
-		havalTransform5 (hcp->digest, hcp->block, hcp->temp);
+	  havalTransform5 (hcp->digest, (mutils_word32 *) (hcp->block), hcp->temp);
 		break;
 	}
 
 	/* fold 256-bit digest to fit the desired hash length (blaargh!): */
-    /* Byte reverse each 32-bit section while outputting if big-endian -- Tines*/
-	switch (hcp->hashLength) {
+	/* Byte reverse each 32-bit section while outputting if big-endian -- Tines*/
+	switch (hcp->hashLength)
+	{
 	case 128:
 		hcp->digest[3] +=
 			( (hcp->digest[7] & 0xFF000000UL)
@@ -858,11 +839,10 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 			| (hcp->digest[4] & 0x0000FF00UL)
 			) >> 8) |
 			( (hcp->digest[7] & 0x000000FFUL) << 24);
-#ifndef WORDS_BIGENDIAN
+#if defined(WORDS_BIGENDIAN)
+		mutils_word32nswap((mutils_word32 *) (hcp->digest), 128/32, MUTILS_TRUE);
+#endif
 		mutils_memcpy(digest, hcp->digest, 128/8);
-#else
-		BYTE_REVERSE(digest, hcp->digest, 128/32);
-#endif /* ?LITTLE_ENDIAN */
 		break;
 	case 160:
 		hcp->digest[4] +=
@@ -877,11 +857,12 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 		hcp->digest[0] +=
 			ROTR
 			((hcp->digest[7] & 0x0000003FUL) | (hcp->digest[6] & 0xFE000000UL) | (hcp->digest[5] & 0x01F80000UL), 19);
-#ifndef WORDS_BIGENDIAN
+
+#if defined(WORDS_BIGENDIAN)
+		mutils_word32nswap((mutils_word32 *) (hcp->digest), 160/32, MUTILS_TRUE);
+#endif
 		mutils_memcpy (digest, hcp->digest, 160/8);
-#else
-	    BYTE_REVERSE(digest, hcp->digest, 160/32);
-#endif /* ?LITTLE_ENDIAN */
+
 		break;
 	case 192:
 		hcp->digest[5] +=
@@ -897,11 +878,12 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 		hcp->digest[0] +=
 			ROTR
 			((hcp->digest[7] & 0x0000001FUL) | (hcp->digest[6] & 0xFC000000UL), 26);
-#ifndef WORDS_BIGENDIAN
+
+#if defined(WORDS_BIGENDIAN)
+		mutils_word32nswap((mutils_word32 *) (hcp->digest), 192/32, MUTILS_TRUE);
+#endif
 		mutils_memcpy (digest, hcp->digest, 192/8);
-#else
-	    BYTE_REVERSE(digest, hcp->digest, 192/32);
-#endif /* ?LITTLE_ENDIAN */
+
 		break;
 	case 224:
 		hcp->digest[6] += (hcp->digest[7]      ) & 0x0000000FUL;
@@ -911,18 +893,20 @@ mutils_error havalFinal (havalContext *hcp, byte *digest)
 		hcp->digest[2] += (hcp->digest[7] >> 18) & 0x0000000FUL;
 		hcp->digest[1] += (hcp->digest[7] >> 22) & 0x0000001FUL;
 		hcp->digest[0] += (hcp->digest[7] >> 27) & 0x0000001FUL;
-#ifndef WORDS_BIGENDIAN
+
+#if defined(WORDS_BIGENDIAN)
+		mutils_word32nswap((mutils_word32 *) (hcp->digest), 224/32, MUTILS_TRUE);
+#endif
 		mutils_memcpy (digest, hcp->digest, 224/8);
-#else
-	    BYTE_REVERSE(digest, hcp->digest, 224/32);
-#endif /* ?LITTLE_ENDIAN */
+
 		break;
 	case 256:
-#ifndef WORDS_BIGENDIAN
+
+#if defined(WORDS_BIGENDIAN)
+		mutils_word32nswap((mutils_word32 *) (hcp->digest), 256/32, MUTILS_TRUE);
+#endif
 		mutils_memcpy (digest, hcp->digest, 256/8);
-#else
-	    BYTE_REVERSE(digest, hcp->digest, 256/32);
-#endif /* ?LITTLE_ENDIAN */
+
 		break;
 	}
 
