@@ -32,6 +32,50 @@
  * to validate inputs.
  */
 
+WIN32DLL_DEFINE
+int mutils_mlock(__const void *addr, __const mutils_word32 len)
+{
+	int ret;
+
+	if ((addr == NULL) || (len == 0))
+	{
+		ret = -1;
+		errno = EINVAL;
+	}
+
+#if defined(HAVE_MLOCK)
+	ret = mlock(addr, len);
+#endif
+
+	return(ret);
+}
+
+WIN32DLL_DEFINE
+int mutils_munlock(__const void *addr, __const mutils_word32 len)
+{
+	int ret;
+
+	if ((addr == NULL) || (len == 0))
+	{
+		ret = -1;
+		errno = EINVAL;
+	}
+
+#if defined(HAVE_MLOCK)
+	ret = munlock(addr, len);
+#endif
+
+	return(ret);
+}
+
+/*
+ * NOTE: In order for memory locking to be useful, we need our own malloc
+ * implementation, as we cannot guarantee that the system-supplied one will
+ * let us know how much memory has been malloc()'ed. For much the same reason,
+ * neither free() nor realloc() can zero safely. This allows the potential
+ * leakage of information.
+ */
+
 WIN32DLL_DEFINE
 void *
 mutils_malloc(__const mutils_word32 n)
@@ -53,6 +97,31 @@ mutils_malloc(__const mutils_word32 n)
 	return(ptr);
 }
 
+WIN32DLL_DEFINE
+void *
+mutils_realloc(__const void *ptr, __const mutils_word32 n)
+{
+	void *ptrOut;
+
+	if ((ptr == NULL) && (n > 0))
+	{
+		ptrOut = mutils_malloc(n);
+	}
+	else
+	{
+		if ((ptr != NULL) && (n == 0))
+		{
+			mutils_free(ptr);
+			ptrOut = NULL;
+		}
+		else
+		{
+		  ptrOut = realloc((void *) ptr, n);
+		}
+	}
+	return(ptrOut);
+}
+
 WIN32DLL_DEFINE
 void
 mutils_free(__const void *ptr)
@@ -85,69 +154,124 @@ mutils_bzero(void *s, __const mutils_word32 n)
 
 WIN32DLL_DEFINE
 void
-mutils_memset(void *s, __const mutils_word8 c, const mutils_word32 n)
+mutils_memset(void *s, __const mutils_word8 c, __const mutils_word32 n)
 {
-	mutils_word8 *stmp = (mutils_word8 *) s;
+#if defined(sparc)
+	/* Sparc needs 8-bit alignment - just use standard memset */
+	memset(s, (int) c, (size_t) n);
+#else
+	mutils_word8 *stmp;
+	mutils_word32 *ltmp = (mutils_word32 *) s;
+	mutils_word32 lump;
 	mutils_word32 i;
+	mutils_word32 words;
+	mutils_word32 remainder;
 
 	if ((s == NULL) || (n == 0))
 	{
 		return;
 	}
 
+	lump = (c << 24) + (c << 16) + (c << 8) + c;
 
-	for (i = 0; i < n; i++, stmp++)
+	words = n >> 2;
+	remainder = n - (words << 2);
+
+	for (i = 0; i < words; i++, ltmp++)
+	{
+		*ltmp = lump;
+	}
+
+	stmp = (mutils_word8 *) ltmp;
+
+	for (i = 0; i < remainder; i++, stmp++)
 	{
 		*stmp = c;
 	}
+#endif
+}
+
+static void
+mutils_memcpy8(mutils_word8 *p_dest, __const mutils_word8 *p_src,
+	__const mutils_word32 n)
+{
+	mutils_word32 i;
+
+	for (i = 0; i < n; i++)
+	{
+		*p_dest++ = *p_src++;
+	}
 }
 
 WIN32DLL_DEFINE
 void
-mutils_memcpy(void *dest, __const void *src, const mutils_word32 n)
+mutils_memcpy(void *dest, __const void *src, __const mutils_word32 n)
 {
 	mutils_word8 *ptr1;
 	mutils_word8 *ptr2;
+	mutils_word32 *bigptr1;
+	mutils_word32 *bigptr2;
 	mutils_word32 i;
+	mutils_word32 words;
+	mutils_word32 remainder;
 
 	if ((dest == NULL) || (src == NULL) || (n == 0))
 	{
 		return;
 	}
 
-	ptr1 = (mutils_word8 *) dest;
-	ptr2 = (mutils_word8 *) src;
+	ptr1 = (mutils_word8 *)src;
+	ptr2 = dest;
 
-	for (i = 0; i < n; i++, ptr1++, ptr2++)
+	/* copy byte-by-byte for small and/or unaligned copies */
+	if ((n < 16) || ((mutils_word32)ptr1 & 0x3) || ((mutils_word32)ptr2 
+& 0x3))
+	{
+		mutils_memcpy8(ptr2, ptr1, n);
+		return;
+	}
+
+	words = n >> 2;
+	remainder = n - (words << 2);
+
+	bigptr1 = (mutils_word32 *) dest;
+	bigptr2 = (mutils_word32 *) src;
+
+	for (i = 0; i < words; i ++, bigptr1++, bigptr2++)
+	{
+		*bigptr1 = *bigptr2;
+	}
+
+	ptr1 = (mutils_word8 *) bigptr1;
+	ptr2 = (mutils_word8 *) bigptr2;
+
+	for (i = 0; i < remainder; i++, ptr1++, ptr2++)
 	{
 		*ptr1 = *ptr2;
 	}
 }
 
-#define MIX32(a) \
-        (((mutils_word32)((mutils_word8 *)(a))[0]) | \
-        (((mutils_word32)((mutils_word8 *)(a))[1]) << 8)| \
-        (((mutils_word32)((mutils_word8 *)(a))[2]) << 16)| \
-        (((mutils_word32)((mutils_word8 *)(a))[3]) << 24))
-
 /*
-   Byte swap a 32bit integer 
+ * If the system ordering is big-endian, convert to little-endian format.
+ * In both cases, the data is held in a temporary workspace that will be
+ * overwritten the next time this function is called. This function is
+ * deprecated and should NOT be called in new code. The ordering macros
+ * should be used instead, for all single-word conversions.
  */
 WIN32DLL_DEFINE
 mutils_word32
 mutils_word32swap(mutils_word32 x)
 {
-#if defined(WORDS_BIGENDIAN)
-	mutils_word32 out = MIX32(x);
+	static mutils_word32 value;
 
-	return(out);
-#else
-	return(x);
-#endif
+	value = mutils_lend32(x);
+	return(value);
 }
 
 /*
-   Byte swap a series of 32-bit integers
+ * Byte swap a series of 32-bit integers. If destructive is set to false, the
+ * output will be placed in a freshly malloc()'ed buffer and the original
+ * data will remain intact.
  */
 WIN32DLL_DEFINE
 mutils_word32 *
@@ -155,16 +279,17 @@ mutils_word32nswap(mutils_word32 *x, mutils_word32 n, mutils_boolean destructive
 {
 	mutils_word32 loop;
 	mutils_word32 *buffer;
-	mutils_word32 *ptr;
+	mutils_word32 *ptrIn;
+	mutils_word32 *ptrOut;
+	mutils_word32 count = n * 4;
 
 	if (destructive == MUTILS_FALSE)
 	{
-		buffer = mutils_malloc(n * 4);
+		buffer = mutils_malloc(count);
 		if (buffer == NULL)
 		{
 			return(NULL);
 		}
-		mutils_memcpy(buffer, x, n * 4);
 	}
 	else
 	{
@@ -172,16 +297,14 @@ mutils_word32nswap(mutils_word32 *x, mutils_word32 n, mutils_boolean destructive
 	}
 
 /*
- * Even though this doesn't do anything for little-endian machines, the non-destructive
- * version is intended to always return a fresh buffer, NOT the original buffer.
+ * This loop is totally useless for destructive processing of little-endian
+ * data on a little-endian machine.
  */
 
-#if defined(WORDS_BIGENDIAN)
-	for (loop = 0, ptr = buffer; loop < n; loop++, ptr++)
+	for (loop = 0, ptrIn = x, ptrOut = buffer; loop < n; loop++, ptrOut++, ptrIn++)
 	{
-		*ptr = MIX32(*ptr);
+		*ptrOut = mutils_lend32(*ptrIn);
 	}
-#endif
 
 	return(buffer);
 }
@@ -193,16 +316,31 @@ mutils_memmove(void *dest, __const void *src, const mutils_word32 n)
 	mutils_word8 *ptr1;
 	mutils_word8 *ptr2;
 	mutils_word32 i;
+	mutils_word32 *bigptr1;
+	mutils_word32 *bigptr2;
+	mutils_word32 words;
+	mutils_word32 remainder;
 
 	if ((dest == NULL) || (src == NULL) || (n == 0))
 	{
 		return;
 	}
 
-	ptr1 = (mutils_word8 *) dest;
-	ptr2 = (mutils_word8 *) src;
+	bigptr1 = (mutils_word32 *) dest;
+	bigptr2 = (mutils_word32 *) src;
 
-	for (i = 0; i < n; i++, ptr1++, ptr2++)
+	words = n >> 2;
+	remainder = n - (words << 2);
+
+	for (i = 0; i < words; i++, bigptr1++, bigptr2++)
+	{
+		*bigptr1 = *bigptr2;
+	}
+
+	ptr1 = (mutils_word8 *) bigptr1;
+	ptr2 = (mutils_word8 *) bigptr2;
+
+	for (i = 0; i < remainder; i++, ptr1++, ptr2++)
 	{
 		*ptr1 = *ptr2;
 	}
@@ -296,6 +434,9 @@ WIN32DLL_DEFINE
 mutils_word8 *
 mutils_strcat(mutils_word8 *dest, __const mutils_word8 *src)
 {
+	mutils_word8 *ptrIn = (mutils_word8 *) src;
+	mutils_word8 *ptrOut = dest + mutils_strlen(dest);
+
 	if (dest == NULL)
 	{
 		return(NULL);
@@ -303,8 +444,15 @@ mutils_strcat(mutils_word8 *dest, __const mutils_word8 *src)
 	if (src == NULL)
 	{
 		return(dest);
+	}	
+	while (*src != 0)
+	{
+		*ptrOut = *ptrIn;
+		ptrOut++;
+		ptrIn++;
 	}
-	return((mutils_word8 *) strcat((char *) dest, (char *) src));
+	*ptrOut = '\0';
+	return((mutils_word8 *) dest);
 }
 
 WIN32DLL_DEFINE
@@ -362,7 +510,7 @@ mutils_strncmp(__const mutils_word8 *src1, const mutils_word8 *src2, const mutil
 	}
 	if (src1 == NULL)
 	{
-		if (src2 = NULL)
+		if (src2 == NULL)
 		{
 			return(0);
 		}
@@ -388,25 +536,10 @@ mutils_val2char(mutils_word8 x)
 {
 	mutils_word8 out;
 
-	switch(x)
-	{
-		case 0x0 : { out = '0'; break; }
-		case 0x1 : { out = '1'; break; }
-		case 0x2 : { out = '2'; break; }
-		case 0x3 : { out = '3'; break; }
-		case 0x4 : { out = '4'; break; }
-		case 0x5 : { out = '5'; break; }
-		case 0x6 : { out = '6'; break; }
-		case 0x7 : { out = '7'; break; }
-		case 0x8 : { out = '8'; break; }
-		case 0x9 : { out = '9'; break; }
-		case 0xa : { out = 'a'; break; }
-		case 0xb : { out = 'b'; break; }
-		case 0xc : { out = 'c'; break; }
-		case 0xd : { out = 'd'; break; }
-		case 0xe : { out = 'e'; break; }
-		case 0xf : { out = 'f'; break; }
-	}
+	static mutils_word8 *table = (mutils_word8 *) "0123456789abcdef";
+
+	out = *(table + x);
+
 	return(out);
 }
 
@@ -434,8 +567,6 @@ mutils_thequals(mutils_word8 *text, mutils_word8 *hash, __const mutils_word32 le
 	mutils_word8  *ptrText = text;
 	mutils_word8  *ptrHash = hash;
 	mutils_word32  loop;
-	mutils_word8   temp;
-	mutils_boolean equals;
 
 	for (loop = 0; loop < len; loop++, ptrHash++)
 	{
@@ -450,3 +581,4 @@ mutils_thequals(mutils_word8 *text, mutils_word8 *hash, __const mutils_word32 le
 	}
 	return(MUTILS_TRUE);
 }
+
